@@ -17,19 +17,24 @@ See the License for the specific language governing permissions and
 from functools import wraps
 import logging
 
-from flask import abort, current_app, g, redirect
+from flask import abort, current_app, url_for
+from flask import redirect as _redirect
 
-__version__ = u'0.1'
+__version__ = u'0.4dev'
 
 log = logging.getLogger(u'flask-featureflags')
 
 RAISE_ERROR_ON_MISSING_FEATURES = u'RAISE_ERROR_ON_MISSING_FEATURES'
 FEATURE_FLAGS_CONFIG = u'FEATURE_FLAGS'
 
+EXTENSION_NAME = "FeatureFlags"
+
+
 class StopCheckingFeatureFlags(Exception):
   """ Raise this inside of a feature flag handler to immediately return False and stop any further handers from running """
   pass
 
+
 def AppConfigFlagHandler(feature=None):
   """ This is the default handler. It checks for feature flags in the current app's configuration.
 
@@ -51,7 +56,7 @@ def AppConfigFlagHandler(feature=None):
     }
 
    """
-  if current_app is None:
+  if not current_app:
     log.warn(u"Got a request to check for {feature} but we're outside the request context. Returning False".format(feature=feature))
     return False
 
@@ -64,9 +69,12 @@ def AppConfigFlagHandler(feature=None):
       log.info(u"No feature flag defined for {feature}".format(feature=feature))
       return False
 
+
 class FeatureFlag(object):
 
-  def __init__(self, app):
+  JINJA_TEST_NAME = u'active_feature'
+
+  def __init__(self, app=None):
     if app is not None:
       self.init_app(app)
 
@@ -74,13 +82,21 @@ class FeatureFlag(object):
     self.handlers = [AppConfigFlagHandler]
 
   def init_app(self, app):
-    """ Inject ourself into the request setup and add a jinja function test """
-    app.before_request(self.process_request)
-    app.jinja_env.tests[u'active_feature'] = self.check
+    """ Add ourselves into the app config and setup, and add a jinja function test """
 
-  def process_request(self):
-    """ Load ourselves into the globals """
-    g.feature_flags = self
+    app.config.setdefault(FEATURE_FLAGS_CONFIG, {})
+    app.config.setdefault(RAISE_ERROR_ON_MISSING_FEATURES, False)
+
+    if hasattr(app, "add_template_test"):
+      # flask 0.10 and higher has a proper hook
+      app.add_template_test(self.check, name=self.JINJA_TEST_NAME)
+    else:
+      app.jinja_env.tests[self.JINJA_TEST_NAME] = self.check
+
+
+    if not hasattr(app, 'extensions'):
+      app.extensions = {}
+    app.extensions[EXTENSION_NAME] = self
 
   def clear_handlers(self):
     """ Clear all handlers. This effectively turns every feature off."""
@@ -94,7 +110,7 @@ class FeatureFlag(object):
     """ Remove a handler from the chain of handlers.  """
     try:
       self.handlers.remove(function)
-    except ValueError: # handler wasn't in the list, pretend we don't notice
+    except ValueError:  # handler wasn't in the list, pretend we don't notice
       pass
 
   def check(self, feature):
@@ -105,7 +121,8 @@ class FeatureFlag(object):
     If you want to a handler to return False and stop the chain, raise the StopCheckingFeatureFlags exception."""
     for handler in self.handlers:
       try:
-        if handler(feature): return True
+        if handler(feature):
+           return True
       except StopCheckingFeatureFlags:
         return False
     else:
@@ -114,13 +131,20 @@ class FeatureFlag(object):
 
 def is_active(feature):
   """ Check if a feature is active """
-  if hasattr(g, u'feature_flags') and isinstance(g.feature_flags, FeatureFlag):
-    return g.feature_flags.check(feature)
+
+  if current_app:
+    feature_flagger = current_app.extensions.get(EXTENSION_NAME)
+    if feature_flagger:
+      return feature_flagger.check(feature)
+    else:
+      raise AssertionError("Oops. This application doesn't have the Flask-FeatureFlag extention installed.")
+
   else:
-    log.warn(u'Got a request to check for {feature} but no handlers are configured. Check your setup. Returning False'.format(feature=feature))
+    log.warn(u"Got a request to check for {feature} but we're running outside the request context. Check your setup. Returning False".format(feature=feature))
     return False
 
-def is_active_feature(feature, redirect_to=None):
+
+def is_active_feature(feature, redirect_to=None, redirect=None):
   """
   Decorator for Flask views. If a feature is off, it can either return a 404 or redirect to a URL if you'd rather.
   """
@@ -129,9 +153,13 @@ def is_active_feature(feature, redirect_to=None):
     def wrapped(*args, **kwargs):
 
       if not is_active(feature):
-        if redirect_to:
-          log.debug(u'Feature {feature} is off, redirecting to {url}'.format(feature=feature, url=redirect_to))
-          return redirect(redirect_to, code=302)
+        url = redirect_to
+        if redirect:
+          url = url_for(redirect)
+
+        if url:
+          log.debug(u'Feature {feature} is off, redirecting to {url}'.format(feature=feature, url=url))
+          return _redirect(url, code=302)
         else:
           log.debug(u'Feature {feature} is off, aborting request'.format(feature=feature))
           abort(404)
@@ -140,9 +168,11 @@ def is_active_feature(feature, redirect_to=None):
     return wrapped
   return _is_active_feature
 
+
 # Silence that annoying No handlers could be found for logger "flask-featureflags"
 class NullHandler(logging.Handler):
   def emit(self, record):
     pass
 
-log.addHandler(NullHandler())
\ No newline at end of file
+
+log.addHandler(NullHandler())
