@@ -2,15 +2,16 @@ from __future__ import with_statement
 
 import unittest
 
-from flask import url_for
+from flask import Flask, url_for
 from .fixtures import app, feature_setup, FEATURE_NAME, FEATURE_IS_ON, FEATURE_IS_OFF, FLAG_CONFIG, RAISE_ERROR
 
 import flask_featureflags as feature_flags
 
+
 class TestFeatureFlagCoreFunctionality(unittest.TestCase):
 
   def setUp(self):
-    app.config[FLAG_CONFIG] = { FEATURE_NAME : True}
+    app.config[FLAG_CONFIG] = {FEATURE_NAME: True}
     app.config['TESTING'] = True
 
     if RAISE_ERROR in app.config:
@@ -45,6 +46,17 @@ class TestFeatureFlagCoreFunctionality(unittest.TestCase):
       assert response.status_code == 404, u'Unexpected status code %s' % response.status_code
       assert FEATURE_IS_ON not in response.data.decode(u'utf-8')
 
+  def test_decorator_redirects_to_url_if_redirect_to_is_set_and_feature_is_off(self):
+      with self.app.test_request_context('/'):
+        url = url_for('redirect_to_with_decorator')
+
+        app.config[FLAG_CONFIG][FEATURE_NAME] = False
+
+        response = self.test_client.get(url)
+        assert response.status_code == 302, u'Unexpected status code %s' % response.status_code
+        assert response.location == url_for('redirect_destination', _external=True), \
+            u'Expected redirect to %s, got %s => ' % (url_for('redirect_destination'), response.location)
+
   def test_decorator_redirects_to_url_if_redirect_is_set_and_feature_is_off(self):
       with self.app.test_request_context('/'):
         url = url_for('redirect_with_decorator')
@@ -53,7 +65,8 @@ class TestFeatureFlagCoreFunctionality(unittest.TestCase):
 
         response = self.test_client.get(url)
         assert response.status_code == 302, u'Unexpected status code %s' % response.status_code
-        assert response.location == url_for('redirect_destination', _external=True), u'Expected redirect to %s, got %s => ' % (url_for('redirect_destination'), response.location)
+        assert response.location == url_for('redirect_destination', _external=True), \
+            u'Expected redirect to %s, got %s => ' % (url_for('redirect_destination'), response.location)
 
   def test_view_based_feature_flag_returns_new_code_if_flag_is_on(self):
     with self.app.test_request_context('/'):
@@ -116,7 +129,7 @@ class TestFeatureFlagCoreFunctionality(unittest.TestCase):
 
     try:
       self.test_client.get(url)
-    except KeyError: # assertRaises no worky for some reason :/
+    except KeyError:  # assertRaises no worky for some reason :/
       pass
 
   def test_do_not_raise_exception_if_we_are_not_in_dev_but_feature_is_missing_and_config_flag_is_set(self):
@@ -154,10 +167,9 @@ class TestFeatureFlagCoreFunctionality(unittest.TestCase):
 
     try:
       self.test_client.get(url)
-    except KeyError: # assertRaises no worky for some reason :/
+    except KeyError:  # assertRaises no worky for some reason :/
       pass
 
-
   def test_do_not_raise_exception_if_we_are_not_in_dev_but_config_is_missing_and_config_flag_is_set(self):
     """If the config section doesn't exist, only raise an error if we're in dev, no matter what the config says """
     with self.app.test_request_context('/'):
@@ -169,4 +181,21 @@ class TestFeatureFlagCoreFunctionality(unittest.TestCase):
 
       response = self.test_client.get(url)
       assert response.status_code == 200, u'Unexpected status code %s' % response.status_code
-      assert FEATURE_IS_OFF in response.data.decode(u'utf-8')
\ No newline at end of file
+      assert FEATURE_IS_OFF in response.data.decode(u'utf-8')
+
+
+class TestAppFactory(unittest.TestCase):
+
+  def test_deferred_initialization_works(self):
+
+    test_app = Flask(__name__)
+    test_app.config[FLAG_CONFIG] = {FEATURE_NAME: True}
+
+    feature_flagger = feature_flags.FeatureFlag()
+    feature_flagger.init_app(test_app)
+
+    with test_app.test_request_context("/"):
+      # Test a couple basic cases to make sure we're actually exercising the code
+      # when we make these calls
+      self.assertTrue(feature_flags.is_active(FEATURE_NAME))
+      self.assertFalse(feature_flags.is_active("DOES_NOT_EXIST"))
